xquery version "3.1";

(:~
 : This module provides the AnnotationAPI for the EMo sample page.
 : It implements the generic AnnotationAPI based on TEI files.
 :
 : @author Michelle Weidling
 : @version 1.0.0
 : @since 0.0.0
 : :)

module namespace aapi="http://emo-viewer.sub.uni-goettingen.de/ns/aapi";

declare namespace output="http://www.w3.org/2010/xslt-xquery-serialization";

import module namespace commons="http://emo-viewer.sub.uni-goettingen.de/ns/commons" at "commons.xqm";
import module namespace requestr="http://exquery.org/ns/request";
import module namespace rest="http://exquery.org/ns/restxq";

declare variable $aapi:server :=
    if(requestr:hostname() = "existdb") then
        doc("../expath-pkg.xml")/*/@name
        => replace("/$", "")
    else
        "http://localhost:8094/exist/restxq";
    
declare
    %rest:GET
    %rest:HEAD
    %rest:path("/sampledata/collection/annotationCollection.json")
    %output:method("json")
function aapi:collection-annotationCollection() {
    local:make-response($commons:sampledata-dir || "/annotationCollection.json")
};

declare
    %rest:GET
    %rest:HEAD
    %rest:path("/sampledata/collection/{$manifest}/annotationPage.json")
    %output:method("json")
function aapi:manifest-annotationPage($manifest as xs:string) {
    local:make-response($commons:sampledata-dir || "/" || $manifest || "-annotationPage.json")
};

declare
    %rest:GET
    %rest:HEAD
    %rest:path("/sampledata/collection/{$manifest}/annotationCollection.json")
    %output:method("json")
function aapi:manifest-annotationCollection($manifest as xs:string) {
    local:make-response($commons:sampledata-dir || "/" || $manifest || "/annotationCollection.json")
};

declare
    %rest:GET
    %rest:HEAD
    %rest:path("/sampledata/collection/{$manifest}/{$item}/annotationCollection.json")
    %output:method("json")
function aapi:item-annotationCollection($manifest as xs:string, $item as xs:string) {
    local:make-response($commons:sampledata-dir || "/" || $manifest || "/" || $item || "/latest/annotationCollection.json")
};

declare
    %rest:GET
    %rest:HEAD
    %rest:path("/sampledata/collection/{$manifest}/{$item}/annotationPage.json")
    %output:method("json")
function aapi:item-annotationPage($manifest as xs:string, $item as xs:string) {
    local:make-response($commons:sampledata-dir || "/" || $manifest || "/" || $item || "/latest/annotationPage.json")
};

declare function local:make-response($path-to-file as xs:string) as item()+ {
    $commons:responseHeader200,
    local:open-and-process-file($path-to-file)
};

declare function local:open-and-process-file($path-to-file as xs:string) as item() {
    util:binary-doc($path-to-file)
    => util:base64-decode()
    => commons:add-server-as-link-prefix($aapi:server)
};
