xquery version "3.1";

(:~
 : This module provides the TextAPI for the EMo sample page.
 : It implements the generic TextAPI based on TEI files.
 :
 : @author Mathias Göbel
 : @author Michelle Weidling
 : @version 1.0.0
 : @since 0.0.0
 : @see https://subugoe.pages.gwdg.de/emo/text-api/page/specs/
 : :)

module namespace textapi="http://emo-viewer.sub.uni-goettingen.de/ns/textapi";

declare namespace output="http://www.w3.org/2010/xslt-xquery-serialization";

import module namespace commons="http://emo-viewer.sub.uni-goettingen.de/ns/commons" at "commons.xqm";
import module namespace requestr="http://exquery.org/ns/request";
import module namespace rest="http://exquery.org/ns/restxq";

declare variable $textapi:server := if(requestr:hostname() = "existdb") then doc("../expath-pkg.xml")/*/@name => replace("/$", "") else "http://localhost:8094/exist/restxq";
    
declare
    %rest:GET
    %rest:HEAD
    %rest:path("/sampledata/collection/collection.json")
    %output:method("json")
function textapi:collection() {
    if (util:binary-doc-available($commons:sampledata-dir || "/collection.json")) then
        (
            $commons:responseHeader200,
            util:binary-doc($commons:sampledata-dir || "/collection.json") => util:base64-decode()
            => commons:add-server-as-link-prefix($textapi:server)
        )
    else
        (
            $commons:responseHeader404,
            error("TEXTAPI01", "Collection 'sampledata' couldn't be found.")
        )
};
    
    
declare
    %rest:GET
    %rest:HEAD
    %rest:path("/sampledata/collection/{$manifest}/full.json")
    %output:method("json")
function textapi:full-item($manifest as xs:string) as item()+ {
    $commons:responseHeader200,
    textapi:get-full($manifest)
};    
    
declare
    %rest:GET
    %rest:HEAD
    %rest:path("/sampledata/collection/{$manifest}/manifest.json")
    %output:method("json")
function textapi:manifest($manifest as xs:string) as item()+ {
    $commons:responseHeader200,
    textapi:get-manifest($manifest)
};

declare function textapi:get-manifest($manifest as xs:string) as item() {
    if (util:binary-doc-available($commons:sampledata-dir || "/" || $manifest || "/manifest.json")) then
        util:binary-doc($commons:sampledata-dir || "/" || $manifest || "/manifest.json")
        => util:base64-decode()
        => commons:add-server-as-link-prefix($textapi:server)
    else
        error("TEXTAPI02", "Requested manifest " || $manifest || " not found.")
};


declare
    %rest:GET
    %rest:HEAD
    %rest:path("/sampledata/collection/{$manifest}-{$item}/latest/item.json")
    %output:method("json")
function textapi:item($manifest as xs:string, $item as xs:string) as item()+ {
    $commons:responseHeader200,
    textapi:get-item($manifest, $item)
};


declare function textapi:get-item($manifest as xs:string, $item as xs:string)
as item() {
    
    util:binary-doc($commons:sampledata-dir || "/" || $manifest || "/" || $item ||
        "/latest/item.json")
    => util:base64-decode()
    => commons:add-server-as-link-prefix($textapi:server)
};


declare function textapi:get-full($manifest as xs:string) as item() {
    json-doc($commons:sampledata-dir || "/" || $manifest || "/full/latest/full.json")
};

declare
    %rest:GET
    %rest:HEAD
    %rest:path("/sampledata/collection/{$manifest}-{$item}/latest/content.html")
    %output:method("xml")
    %output:indent("no")
function textapi:content($manifest as xs:string, $item as xs:string)
as item()+ {
    $commons:responseHeader200,
    textapi:get-content($manifest, $item)
};


declare function textapi:get-content($manifest as xs:string, $item as xs:string) {
    doc($commons:sampledata-dir || "/" || $manifest || "/" || $item ||
        "/latest/content.html")
};
