xquery version "3.1";

(:~
 : Providing an API endpoint for triggering test execution.
 : This endpoint has been established instead of the text execution in post-install.xq
 : since at this point the RESTXQ API isn't fired up yet which causes the tests to throw errors.
 : 
 : @author Michelle Weidling
 : @version 1.0.0
 : @since 0.0.0
 :)

module namespace trigger="http://emo-viewer.sub.uni-goettingen.de/ns/trigger";

import module namespace aapi-tests="http://emo-viewer.sub.uni-goettingen.de/ns/aapi/tests" at "../tests/aapi-tests.xqm";
import module namespace aux-tests="http://emo-viewer.sub.uni-goettingen.de/ns/aux/tests" at "../tests/aux-tests.xqm";
import module namespace commons-tests="http://emo-viewer.sub.uni-goettingen.de/ns/commons/tests" at "../tests/commons-tests.xqm";
import module namespace rest="http://exquery.org/ns/restxq";
import module namespace tapi-tests="http://emo-viewer.sub.uni-goettingen.de/ns/textapi/tests"  at "../tests/textapi-tests.xqm";
import module namespace test="http://exist-db.org/xquery/xqsuite" at "resource:org/exist/xquery/lib/xqsuite/xqsuite.xql";

(:~
 : Triggers the tests for the EMo Viewer backend. Called by the CI.
 : 
 : @param $token A CI token
 : @return item() A log message to std out. In the Docker environment, this goes to exist.log.
 : @error The CI token provided is incorrect
 :)
declare
  %rest:GET
  %rest:HEAD
  %rest:path("/trigger-tests")
  %rest:query-param("token", "{$token}")
function trigger:trigger($token)
as item()? {
  if( $token ne environment-variable("EXIST_TESTING_PW") )
    then error(QName("error://1", "trigger-tests"), "Token for triggering API tests incorrect.")
  else
    let $sysout := util:log-system-out("Package installation done. Running tests…")
    let $tests := 
        (
            test:suite(util:list-functions("http://emo-viewer.sub.uni-goettingen.de/ns/aux/tests")),
            test:suite(util:list-functions("http://emo-viewer.sub.uni-goettingen.de/ns/commons/tests")),
            test:suite(util:list-functions("http://emo-viewer.sub.uni-goettingen.de/ns/textapi/tests")),
            test:suite(util:list-functions("http://emo-viewer.sub.uni-goettingen.de/ns/aapi/tests"))
        )
    
    let $fileSeparator := util:system-property("file.separator")
    let $system-path := system:get-exist-home() || $fileSeparator
    
    let $testWrap := <tests time="{current-dateTime()}">{ $tests }</tests>
    
    let $filename := $system-path || "test-results.xml"
    let $file := file:serialize($testWrap, $filename, ())
    
    return
        util:log-system-out("Tests complete. See " || $filename)
};