xquery version "3.1";

(:~
 : Test module for the aux module used for testing.
 : 
 : @author Michelle Weidling
 : @version 0.1.0
 :)

module namespace aux-tests="http://emo-viewer.sub.uni-goettingen.de/ns/aux/tests";

declare namespace http = "http://expath.org/ns/http-client";
declare namespace tei="http://www.tei-c.org/ns/1.0";

declare variable $aux-tests:restxq := "http://0.0.0.0:8080/exist/restxq";

import module namespace map="http://www.w3.org/2005/xpath-functions/map";
import module namespace test="http://exist-db.org/xquery/xqsuite" at "resource:org/exist/xquery/lib/xqsuite/xqsuite.xql";
import module namespace aux="http://emo-viewer.sub.uni-goettingen.de/ns/aux" at "aux.xqm";

declare
    %test:assertTrue
function aux-tests:is-endpoint-http200() {
    aux:is-endpoint-http200("http://0.0.0.0:8080/exist/")
};

declare
    %test:assertFalse
function aux-tests:is-endpoint-not-http200() {
    aux:is-endpoint-http200("http://0.0.0.0:8080/some-test-url/")
};

declare
    %test:assertEquals("200")
function aux-tests:get-http-status() {
    aux:get-http-status("http://0.0.0.0:8080/exist/")
};

declare
    %test:assertEquals("404")
function aux-tests:get-http-status-failure() {
    aux:get-http-status("http://0.0.0.0:8080/exist/restxq")
};

declare
    %test:args("collection.json")
    %test:assertXPath("map:get($result, 'description') = 'This collection has been prepared for testing purposes.'")
    %test:args("document1/manifest.json")
    %test:assertXPath("map:get($result, 'label') = 'Testdata Manifest 001'")
    %test:args("document1-1/latest/item.json")
    %test:assertXPath("map:get($result, 'type') = 'page'")
function aux-tests:get-json-from-url-TextAPI($ep as xs:string) {
    aux:get-json-from-url($aux:restxq || $ep)
};

declare
    %test:args("annotationCollection.json")
    %test:assertXPath("map:get($result, 'total') = 5")
    %test:args("document1/annotationCollection.json")
    %test:assertXPath("map:get($result, 'label') = 'Annotations for Document 1, page 1'")
    %test:args("document1/annotationPage.json")
    %test:assertXPath("map:get($result, 'id') = 'http://emo-viewer.sub.uni-goettingen.de/ns/annotations/annotationPage/document1'")
    %test:args("document1/1/annotationCollection.json")
    %test:assertXPath("map:get($result, 'id') = 'http://emo-viewer.sub.uni-goettingen.de/ns/annotations/annotationCollection/document1-1'")
    %test:args("document1/1/annotationPage.json")
    %test:assertXPath("map:get($result, 'id') = 'http://emo-viewer.sub.uni-goettingen.de/ns/annotations/annotationPage/document1-1'")
function aux-tests:get-json-from-url-AnnotationAPI($ep as xs:string) {
    aux:get-json-from-url($aux:restxq || $ep)
};

declare
    %test:assertXPath("$result/@href = 'http://0.0.0.0:8080/exist/'")
function aux-tests:make-request() {
    aux:make-request("http://0.0.0.0:8080/exist/")
};

declare
    %test:args("", "", "collection.json")
    %test:assertEquals("/db/apps/emo-viewer/data/testData/collection.json")
    %test:args("", "", "manifest.json")
    %test:assertEquals("/db/apps/emo-viewer/data/testData/manifest.json")
    %test:args("", "", "item.json")
    %test:assertEquals("/db/apps/emo-viewer/data/testData/item.json")
function aux-tests:make-db-location-TextAPI($prefix as xs:string?,
$item as xs:string?, $resource-item as xs:string) as xs:string {
    aux:make-testData-location($prefix, $item, $resource-item)
};

declare
    %test:args("collection", "", "annotationCollection.json")
    %test:assertEquals("/db/apps/emo-viewer/data/testData/collection-annotationCollection.json")
    %test:args("document1", "", "annotationCollection.json")
    %test:assertEquals("/db/apps/emo-viewer/data/testData/document1-annotationCollection.json")
    %test:args("document1", "1", "annotationCollection.json")
    %test:assertEquals("/db/apps/emo-viewer/data/testData/document1-1-annotationCollection.json")
function aux-tests:make-db-location-AnnotationAPI-annotationCollection($prefix as xs:string?,
$item as xs:string?, $resource-item as xs:string) as xs:string {
    aux:make-testData-location($prefix, $item, $resource-item)
};

declare
    %test:args("collection", "", "annotationPage.json")
    %test:assertEquals("/db/apps/emo-viewer/data/testData/collection-annotationPage.json")
    %test:args("document1", "", "annotationPage.json")
    %test:assertEquals("/db/apps/emo-viewer/data/testData/document1-annotationPage.json")
    %test:args("document1", "1", "annotationPage.json")
    %test:assertEquals("/db/apps/emo-viewer/data/testData/document1-1-annotationPage.json")
function aux-tests:make-db-location-AnnotationAPI-annotationPage($prefix as xs:string?,
$item as xs:string?, $resource-item as xs:string) as xs:string {
    aux:make-testData-location($prefix, $item, $resource-item)
};

declare
    %test:args("", "", "collection")
    %test:assertTrue
    %test:args("document1", "", "manifest")
    %test:assertTrue
    %test:args("document1", "1", "item")
    %test:assertTrue
function aux-tests:are-endpoint-data-and-control-file-equal-TextAPI($manifest as xs:string?,
$item as xs:string?, $resource-type as xs:string) as xs:boolean {
    aux:are-endpoint-data-and-control-file-equal($manifest, $item, $resource-type)
};

declare
    %test:args("collection", "", "annotationCollection")
    %test:assertTrue
    %test:args("document1", "", "annotationCollection")
    %test:assertTrue
    %test:args("document1", "1", "annotationCollection")
    %test:assertTrue
function aux-tests:are-endpoint-data-and-control-file-equal-AnnotationAPI-collection($manifest as xs:string?,
$item as xs:string?, $resource-type as xs:string) as xs:boolean {
    aux:are-endpoint-data-and-control-file-equal($manifest, $item, $resource-type)
};

declare
    %test:assertEquals("http://0.0.0.0:8080/exist/restxq/sampledata/collection/collection.json")
function aux-tests:make-endpoint-url-collection-TextAPI() {
    aux:make-endpoint-url("", "", "collection.json")
};

declare
    %test:assertEquals("http://0.0.0.0:8080/exist/restxq/sampledata/collection/annotationCollection.json")
function aux-tests:make-endpoint-url-collection-AnnotationAPI() {
    aux:make-endpoint-url("collection", "", "annotationCollection.json")
};

declare
    %test:assertEquals("http://0.0.0.0:8080/exist/restxq/sampledata/collection/document1/manifest.json")
function aux-tests:make-endpoint-url-manifest-TextAPI() {
    aux:make-endpoint-url("document1", "", "manifest.json")
};

declare
    %test:args("document1", "", "annotationPage.json")
    %test:assertEquals("http://0.0.0.0:8080/exist/restxq/sampledata/collection/document1/annotationPage.json")
    %test:args("document1", "", "annotationCollection.json")
    %test:assertEquals("http://0.0.0.0:8080/exist/restxq/sampledata/collection/document1/annotationCollection.json")
function aux-tests:make-endpoint-url-manifest-AnnotationAPI($prefix as xs:string,
$item as xs:string, $json-filename as xs:string) {
    aux:make-endpoint-url($prefix, $item, $json-filename)
};

declare
    %test:assertEquals("http://0.0.0.0:8080/exist/restxq/sampledata/collection/document1-1/latest/item.json")
function aux-tests:make-endpoint-url-item-TextAPI() {
    aux:make-endpoint-url("document1", "1", "item.json")
};

declare
    %test:args("document1", "1", "annotationPage.json")
    %test:assertEquals("http://0.0.0.0:8080/exist/restxq/sampledata/collection/document1/1/annotationPage.json")
    %test:args("document1", "1", "annotationCollection.json")
    %test:assertEquals("http://0.0.0.0:8080/exist/restxq/sampledata/collection/document1/1/annotationCollection.json")
function aux-tests:make-endpoint-url-item-AnnotationAPI($prefix as xs:string,
$item as xs:string, $json-filename as xs:string) {
    aux:make-endpoint-url($prefix, $item, $json-filename)
};


declare
    %test:args("document1", "2")
    %test:assertEquals("document1-2-")
    %test:args("collection", "")
    %test:assertEquals("collection-")
    %test:args("", "")
    %test:assertEquals("")
function aux-tests:make-concat-prefix($prefix as xs:string, $item as xs:string)
as xs:string {
    aux:make-concat-prefix($prefix, $item)
};