xquery version "3.1";

(:~
 : Provides functions that are used by both the tests of the TextAPI and the
 : AnnotationAPI.
 : 
 : @author Michelle Weidling
 : @version 1.0.0
 : @since 0.0.0
 :)

module namespace aux="http://emo-viewer.sub.uni-goettingen.de/ns/aux";

declare namespace http = "http://expath.org/ns/http-client";

declare variable $aux:restxq := "http://0.0.0.0:8080/exist/restxq/sampledata/collection/";
declare variable $aux:testdata-coll := "/db/apps/emo-viewer/data/testData/";


declare function aux:is-endpoint-http200($url as xs:string) as xs:boolean {
    let $http-status := aux:get-http-status($url)
    return
        $http-status = "200"
};

declare function aux:get-http-status($url as xs:string) as xs:string {
    let $req := aux:make-request($url)
    return
        http:send-request($req)[1]/@status
};

declare function aux:get-json-from-url($url as xs:string) as item() {
    let $req := aux:make-request($url)
    return
        http:send-request($req)[2]
        => util:base64-decode()
        => parse-json()
};

declare function aux:make-request($url as xs:string) {
    <http:request href="{$url}" method="get">
        <http:header name="Connection" value="close"/>
   </http:request>
};

declare function aux:are-endpoint-data-and-control-file-equal($manifest as xs:string?,
$item as xs:string?, $resource-type as xs:string)  {
    let $json-filename := $resource-type || ".json"
    let $url := aux:make-endpoint-url($manifest, $item, $json-filename)
    let $db-location := aux:make-testData-location($manifest, $item, $json-filename)
    
    let $json-ep := aux:get-json-from-url($url)
    let $json-control := json-doc($db-location)
    return
        deep-equal($json-control, $json-ep)
};

declare function aux:make-endpoint-url($prefix as xs:string?,
$item as xs:string?, $json-filename as xs:string) as xs:string {
    if(matches($json-filename, "annotation")) then
        aux:make-endpoint-url-for-AnnotationAPI($prefix, $item, $json-filename)
    else
        aux:make-endpoint-url-for-TextAPI($prefix, $item, $json-filename)
};

declare function aux:make-endpoint-url-for-TextAPI($prefix as xs:string?,
$item as xs:string?, $json-filename as xs:string) as xs:string {
    if ($item) then
            $aux:restxq || $prefix || "-" || $item || "/latest/" || $json-filename
    else if($prefix != "collection" and $prefix != "") then
        $aux:restxq || $prefix || "/" || $json-filename
    else
        $aux:restxq || $json-filename
};

declare function aux:make-endpoint-url-for-AnnotationAPI($prefix as xs:string?,
$item as xs:string?, $json-filename as xs:string) as xs:string {
    if ($item) then
        $aux:restxq || $prefix || "/" || $item || "/" || $json-filename
    else if($prefix = "collection") then
        $aux:restxq || $json-filename
    else
        $aux:restxq || $prefix || "/" || $json-filename
};


declare function aux:make-testData-location($prefix as xs:string?,
$item as xs:string?, $resource-item as xs:string) as xs:string {
    let $concat-prefix := aux:make-concat-prefix($prefix, $item)
    return
        $aux:testdata-coll || $concat-prefix || $resource-item
};

declare function aux:make-testData-location-TextAPI($prefix as xs:string?,
$item as xs:string?, $resource-item as xs:string) as xs:string {
    let $concat-prefix := aux:make-concat-prefix($prefix, $item)
    return
        $aux:testdata-coll || $concat-prefix || $resource-item
};

declare function aux:make-testData-location-AnnotationAPI($prefix as xs:string?,
$item as xs:string?, $resource-item as xs:string) as xs:string {
    ""
};

declare function aux:make-concat-prefix($prefix as xs:string, $item as xs:string)
as xs:string {
    if ($item) then
        $prefix || "-" || $item || "-"
    else if ($prefix) then
        $prefix || "-"
    else
        ""
};
