xquery version "3.1";

(:~
 : Test module for the RESTXQ endpoints of the sample implementation of the
 : TextAPI.
 : 
 : @author Michelle Weidling
 : @version 0.1.0
 :)

module namespace tapi-tests="http://emo-viewer.sub.uni-goettingen.de/ns/textapi/tests";

declare namespace http = "http://expath.org/ns/http-client";
declare namespace tei="http://www.tei-c.org/ns/1.0";

import module namespace map="http://www.w3.org/2005/xpath-functions/map";
import module namespace textapi="http://emo-viewer.sub.uni-goettingen.de/ns/textapi" at "../modules/textapi.xqm";
import module namespace test="http://exist-db.org/xquery/xqsuite" at "resource:org/exist/xquery/lib/xqsuite/xqsuite.xql";
import module namespace aux="http://emo-viewer.sub.uni-goettingen.de/ns/aux" at "aux.xqm";

declare variable $tapi-tests:restxq := "http://0.0.0.0:8080/exist/restxq";
declare variable $tapi-tests:restxq-sampledata := "http://0.0.0.0:8080/exist/restxq/sampledata/collection";
declare variable $tapi-tests:testdata-coll := "/db/apps/emo-viewer/data/testData";

declare variable $tapi-tests:collection-url := $tapi-tests:restxq-sampledata || "/collection.json";
declare variable $tapi-tests:manifest-url := $tapi-tests:restxq-sampledata || "/document1/manifest.json";
declare variable $tapi-tests:item-url := $tapi-tests:restxq-sampledata || "/document1-1/latest/item.json";
declare variable $tapi-tests:content-url := $tapi-tests:restxq-sampledata || "/document1-1/latest/content.html";
declare variable $tapi-tests:full-url := $tapi-tests:restxq-sampledata || "/document3/full.json";

declare
    %test:assertTrue
function tapi-tests:is-collection-available() as item() {
    aux:is-endpoint-http200($tapi-tests:collection-url)
};


declare
    %test:assertTrue
function tapi-tests:has-collection-right-content() {
    aux:are-endpoint-data-and-control-file-equal("", "", "collection")
};


declare
    %test:assertTrue
function tapi-tests:is-manifest-available() as item() {
    aux:is-endpoint-http200($tapi-tests:manifest-url)
};


declare
    %test:assertTrue
function tapi-tests:has-manifest-right-content() {
    aux:are-endpoint-data-and-control-file-equal("document1", "", "manifest")
};


declare
    %test:assertTrue
function tapi-tests:are-collection-and-manifest-correctly-linked() {
    let $collection := aux:get-json-from-url($tapi-tests:collection-url)
    let $link-to-doc1 := map:get($collection, "sequence")?1 => map:get("id")
    let $manifest-url :=
        replace($link-to-doc1, "localhost:8094", "0.0.0.0:8080")
        => replace("api/", "")
    let $http-status := aux:get-http-status($manifest-url)
    return $link-to-doc1
(:        $http-status = "200":)
};


declare
    %test:assertFalse
function tapi-tests:are-collection-and-manifest-incorrectly-linked() {
    let $collection := aux:get-json-from-url($tapi-tests:collection-url)
    let $link-to-doc1 := map:get($collection, "sequence")?1 => map:get("id")
    let $manifest-url := $tapi-tests:restxq || "/" || $link-to-doc1
    let $http-status := aux:get-http-status($manifest-url)
    return
        $http-status = "200"
};


declare
    %test:assertTrue
function tapi-tests:is-item-available() {
    aux:is-endpoint-http200($tapi-tests:item-url)
};


declare
    %test:assertTrue
function tapi-tests:has-item-right-content() {
    aux:are-endpoint-data-and-control-file-equal("document1", "1", "item")
};


declare
    %test:assertTrue
function tapi-tests:are-manifest-and-item-correctly-linked() {
    let $manifest := aux:get-json-from-url($tapi-tests:manifest-url)
    let $link-to-item1 := map:get($manifest, "sequence")?1 => map:get("id")
    let $item-url :=
        replace($link-to-item1, "localhost:8094", "0.0.0.0:8080")
        => replace("api/", "")
    let $http-status := aux:get-http-status($item-url)
    return
        $http-status = "200"
};

declare
    %test:assertFalse
function tapi-tests:are-manifest-and-item-incorrectly-linked() {
    let $url := $tapi-tests:restxq || "sampledata/document1/manifest.json"
    let $manifest := aux:get-json-from-url($tapi-tests:manifest-url)
    let $link-to-item1 := map:get($manifest, "sequence")?1 => map:get("id")
    let $item-url := $tapi-tests:restxq || "/" || $link-to-item1
    let $http-status := aux:get-http-status($item-url)
    return
        $http-status = "200"
};

declare
    %test:assertTrue
function tapi-tests:is-full-item-available() {
    aux:is-endpoint-http200($tapi-tests:full-url)
};


(:declare:)
(:    %test:assertTrue:)
(:function tapi-tests:has-full-right-content() {:)
(:    aux:are-endpoint-data-and-control-file-equal("full", ""):)
(:};:)


declare
    %test:assertTrue
function tapi-tests:is-content-available() {
    aux:is-endpoint-http200($tapi-tests:content-url)
};


declare
    %test:assertTrue
function tapi-tests:has-content-right-content() {
    let $req := <http:request href="{$tapi-tests:content-url}" method="get">
                <http:header name="Connection" value="close"/>
           </http:request>
    let $result-html := http:send-request($req)[2]
    let $src-html := doc($tapi-tests:testdata-coll || "/content.html")
    return
        deep-equal($src-html, $result-html)
};


declare
    %test:assertTrue
function tapi-tests:is-content-correctly-linked() {
    let $result-json := aux:get-json-from-url($tapi-tests:item-url)
    let $content-link := map:get($result-json, "content")
    let $content-url :=
        replace($content-link, "localhost:8094", "0.0.0.0:8080")
        => replace("api/", "")
    
    let $http-status := aux:get-http-status($content-url)
    return
        $http-status = "200"
};

declare
    %test:assertFalse
function tapi-tests:is-content-incorrectly-linked() {
    let $result-json := aux:get-json-from-url($tapi-tests:item-url)
    let $content-link := map:get($result-json, "content")
    
    let $content-url := $tapi-tests:restxq || "/" || $content-link
    let $http-status := aux:get-http-status($content-url)
    return
        $http-status = "200"
};


declare
    %test:assertXPath("map:contains($result, 'textapi')")
    %test:assertXPath("map:contains($result, 'title')")
    %test:assertXPath("map:contains($result, 'collector')")
    %test:assertXPath("map:contains($result, 'sequence')")
function tapi-tests:collection-are-all-required-fields-set() {
    aux:get-json-from-url($tapi-tests:collection-url)
};


declare
    %test:assertXPath("map:contains($result, 'textapi')")
    %test:assertXPath("map:contains($result, 'id')")
    %test:assertXPath("map:contains($result, 'label')")
    %test:assertXPath("map:contains($result, 'sequence')")
    %test:assertXPath("map:contains($result, 'license')")
function tapi-tests:document1-are-all-required-fields-set() {
    aux:get-json-from-url($tapi-tests:restxq-sampledata || "/document1/manifest.json")
};

declare
    %test:assertXPath("map:contains($result, 'textapi')")
    %test:assertXPath("map:contains($result, 'id')")
    %test:assertXPath("map:contains($result, 'label')")
    %test:assertXPath("map:contains($result, 'sequence')")
    %test:assertXPath("map:contains($result, 'license')")
function tapi-tests:document2-are-all-required-fields-set() {
    aux:get-json-from-url($tapi-tests:restxq-sampledata || "/document2/manifest.json")
};

declare
    %test:assertXPath("map:contains($result, 'textapi')")
    %test:assertXPath("map:contains($result, 'id')")
    %test:assertXPath("map:contains($result, 'label')")
    %test:assertXPath("map:contains($result, 'sequence')")
    %test:assertXPath("map:contains($result, 'license')")
function tapi-tests:document3-are-all-required-fields-set() {
    aux:get-json-from-url($tapi-tests:restxq-sampledata || "/document3/manifest.json")
};

declare
    %test:assertXPath("map:contains($result, 'textapi')")
    %test:assertXPath("map:contains($result, 'title')")
    %test:assertXPath("map:contains($result, 'type')")
    %test:assertXPath("map:contains($result, 'lang')")
    %test:assertXPath("map:contains($result, 'content')")
    %test:assertXPath("map:contains($result, 'content-type')")
function tapi-tests:document1-item1-are-all-required-fields-set() {
    let $url := $tapi-tests:restxq-sampledata || "/document1-1/latest/item.json"
    return
        aux:get-json-from-url($url)
};

declare
    %test:assertXPath("map:contains($result, 'textapi')")
    %test:assertXPath("map:contains($result, 'title')")
    %test:assertXPath("map:contains($result, 'type')")
    %test:assertXPath("map:contains($result, 'lang')")
    %test:assertXPath("map:contains($result, 'content')")
    %test:assertXPath("map:contains($result, 'content-type')")
function tapi-tests:document1-item2-are-all-required-fields-set() {
    let $url := $tapi-tests:restxq-sampledata || "/document1-2/latest/item.json"
    return
        aux:get-json-from-url($url)
};

declare
    %test:assertXPath("map:contains($result, 'textapi')")
    %test:assertXPath("map:contains($result, 'title')")
    %test:assertXPath("map:contains($result, 'type')")
    %test:assertXPath("map:contains($result, 'lang')")
    %test:assertXPath("map:contains($result, 'content')")
    %test:assertXPath("map:contains($result, 'content-type')")
function tapi-tests:document2-item3-are-all-required-fields-set() {
    let $url := $tapi-tests:restxq-sampledata || "/document2-3/latest/item.json"
    return
        aux:get-json-from-url($url)
};

declare
    %test:assertXPath("map:contains($result, 'textapi')")
    %test:assertXPath("map:contains($result, 'title')")
    %test:assertXPath("map:contains($result, 'type')")
    %test:assertXPath("map:contains($result, 'lang')")
    %test:assertXPath("map:contains($result, 'content')")
    %test:assertXPath("map:contains($result, 'content-type')")
function tapi-tests:document2-item4-are-all-required-fields-set() {
    let $url := $tapi-tests:restxq-sampledata || "/document2-4/latest/item.json"
    return
        aux:get-json-from-url($url)
};

declare
    %test:assertXPath("map:contains($result, 'textapi')")
    %test:assertXPath("map:contains($result, 'title')")
    %test:assertXPath("map:contains($result, 'type')")
    %test:assertXPath("map:contains($result, 'lang')")
    %test:assertXPath("map:contains($result, 'content')")
    %test:assertXPath("map:contains($result, 'content-type')")
function tapi-tests:document3-item5-are-all-required-fields-set() {
    let $url := $tapi-tests:restxq-sampledata || "/document3-5/latest/item.json"
    return
        aux:get-json-from-url($url)
};

declare
    %test:assertXPath("map:contains($result, 'textapi')")
    %test:assertXPath("map:contains($result, 'title')")
    %test:assertXPath("map:contains($result, 'type')")
    %test:assertXPath("map:contains($result, 'lang')")
    %test:assertXPath("map:contains($result, 'content')")
    %test:assertXPath("map:contains($result, 'content-type')")
function tapi-tests:document3-full-are-all-required-fields-set() {
    let $url := $tapi-tests:restxq-sampledata || "/document3/full.json"
    return
        aux:get-json-from-url($url)
};

declare
    %test:assertXPath("$result != false()")
function tapi-tests:are-collection-links-prefixed-by-server() {
    let $collection := aux:get-json-from-url($tapi-tests:collection-url)
    let $no-of-manifests := 
        map:get($collection, "sequence")
        => array:size()
    let $manifest-ids :=
        for $iii in 1 to $no-of-manifests return
            map:get($collection, "sequence")?($iii) 
            => map:get("id")
    let $annotationColl-id := map:get($collection, "annotationCollection")
    for $id in ($manifest-ids, $annotationColl-id) return
        starts-with($id, "http")
};

declare
    %test:assertXPath("$result != false()")
function tapi-tests:are-manifest-links-prefixed-by-server() {
    let $collection := aux:get-json-from-url($tapi-tests:manifest-url)
    let $no-of-items := 
        map:get($collection, "sequence")
        => array:size()
    let $item-ids :=
        for $iii in 1 to $no-of-items return
            map:get($collection, "sequence")?($iii) 
            => map:get("id")
    let $support-url := 
        map:get($collection, "support")?(1)
        => map:get("url")
    for $id in ($item-ids, $support-url) return
        starts-with($id, "http")
};


declare
    %test:assertXPath("$result != false()")
function tapi-tests:are-item-links-prefixed-by-server() {
    let $item := aux:get-json-from-url($tapi-tests:item-url)
    let $content := map:get($item, "content")
    let $annotationCollection := map:get($item, "annotationCollection")
    let $image-url := 
        map:get($item, "image")
        => map:get("id")
    for $id in ($content, $annotationCollection, $image-url) return
        starts-with($id, "http")
};

(: TODO: Überlegen, wie man den Fehler für Collections gut testen kann :)
declare
    %test:assertError("TEXTAPI01")
    %test:pending
function tapi-tests:error-collection-not-found() {
    textapi:collection()
};

declare
    %test:assertError("TEXTAPI02")
    %test:pending
function tapi-tests:error-manifest-not-found() {
    let $url := $tapi-tests:restxq-sampledata || "/document4/manifest.json"
    return
        aux:get-json-from-url($url)
};
